#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2017-17-05
#
# Check Point vsx Cluster status
#
# Monitor status of virtual systems in Check Point vsx/vsls cluster
#
# 2018-08-03: changed snmp scan function
# 2018-08-13: code cleanup, add metrics
# 2020-06-08: changed snmp-scan function
# 2021-09-06: rewritten for CMK 2.0
# 2021-09-07: added WATO for check and discovery function
#             renamed from checkpoint_vsx to checkpoint_vsx_system
# 2021-09-21: changed graph definition for packets graph to stack
# 2021-09-24: changed graph definition for connections graph to make it scale with
#             the active connections (suggested by raulcurro[at]gmail[dot]com)
#             moved Main IP to details, if it is not an IPv4 address
# 2022-10-23: fixed warning on upgrade "non-empty params vanished" for policyname and ha_state
# 2023-05-29: moved gui files to ~/local/lib/check_mk/gui/plugins/...
# 2023-06-17: added support for predicted levels for connection active
#             changed bytes/packets metrics to check_levels/check_levels_predictive
# 2023-06-18: cleanup variable names to better meet python standards
# 2023-06-23: fixed _render_per_second to output non scientific format
#             changed _render_per_second to match output in metrics

#
# snmpwalk sample
#
# .1.3.6.1.4.1.2620.1.16.22.1.1.1.3.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.1.16.22.1.1.2.3.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.1.16.22.1.1.3.3.0 = STRING: "HRI"
# .1.3.6.1.4.1.2620.1.16.22.1.1.4.3.0 = STRING: "Virtual System"
# .1.3.6.1.4.1.2620.1.16.22.1.1.5.3.0 = STRING: "10.140.1.3"
# .1.3.6.1.4.1.2620.1.16.22.1.1.6.3.0 = STRING: "Standard"
# .1.3.6.1.4.1.2620.1.16.22.1.1.7.3.0 = STRING: "Active"
# .1.3.6.1.4.1.2620.1.16.22.1.1.8.3.0 = STRING: "Trust established"
# .1.3.6.1.4.1.2620.1.16.22.1.1.9.3.0 = STRING: "Standby"
# .1.3.6.1.4.1.2620.1.16.22.1.1.10.3.0 = Gauge32: 0
#
# .1.3.6.1.4.1.2620.1.16.23.1.1.2.3.0 = Gauge32: 40
# .1.3.6.1.4.1.2620.1.16.23.1.1.3.3.0 = Gauge32: 864
# .1.3.6.1.4.1.2620.1.16.23.1.1.4.3.0 = Gauge32: 14900
# .1.3.6.1.4.1.2620.1.16.23.1.1.5.3.0 = STRING: "69459"
# .1.3.6.1.4.1.2620.1.16.23.1.1.6.3.0 = STRING: "2405"
# .1.3.6.1.4.1.2620.1.16.23.1.1.7.3.0 = STRING: "67054"
# .1.3.6.1.4.1.2620.1.16.23.1.1.8.3.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.16.23.1.1.9.3.0 = STRING: "4228862"
# .1.3.6.1.4.1.2620.1.16.23.1.1.10.3.0 = STRING: "72445"
# .1.3.6.1.4.1.2620.1.16.23.1.1.11.3.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.16.23.1.1.12.3.0 = STRING: "7074"
# .1.3.6.1.4.1.2620.1.16.23.1.1.13.3.0 = INTEGER: 0
#


import ipaddress
import time
from dataclasses import dataclass
from typing import List, Dict, Optional, Any

# import same render function as used for the metrics (undocumented for check API, and metrics as well)
from cmk.utils.render import fmt_number_with_precision

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    State,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
    Metric,
    get_value_store,
    get_rate,
    GetRateError,
    check_levels,
    check_levels_predictive,
    render,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class VsxMetric:
    name: str
    value: float
    label: str
    levels: str
    render_func: Any


@dataclass
class CheckpointVsx:
    vs_id: str
    vs_type: str
    main_ip: str
    policy_name: str
    policy_type: str
    sic_trust_state: str
    ha_state: str
    vs_weight: str
    connections_active: int
    connections_peak: int
    connections_limit: int
    metrics_rate: List[VsxMetric]


# render the /s in the same way as in the metrics
def _render_per_second(value: float) -> str:
    return fmt_number_with_precision(value, drop_zeroes=True, unit="/s")
    # return f'{value:.2f}/s'


def parse_checkpoint_vsx_system(string_table: StringTable) -> Optional[Dict[str, CheckpointVsx]]:
    vsx_systems = {}
    for entry in string_table:
        try:
            vs_id, vs_name, vs_type, main_ip, policy_name, policy_type, sic_trust_state, ha_state, vs_weight, \
                connections_active, connections_peak, connections_limit, packets_processed, packets_dropped, \
                packets_accepted, packets_rejected, bytes_accepted, bytes_dropped, bytes_rejected, loggs_send = entry
        except ValueError:
            return

        vsx_systems[vs_name] = CheckpointVsx(
            vs_id=vs_id,
            vs_type=vs_type,
            main_ip=main_ip,
            policy_name=policy_name,
            policy_type=policy_type,
            sic_trust_state=sic_trust_state,
            ha_state=ha_state,
            vs_weight=vs_weight,
            connections_active=int(connections_active),
            connections_peak=int(connections_peak),
            connections_limit=int(connections_limit),
            metrics_rate=[
                VsxMetric(
                    name='bytes_accepted',
                    value=float(bytes_accepted),
                    label='Bytes accepted',
                    levels='levels_bytes_accepted',
                    render_func=render.iobandwidth,
                ),
                VsxMetric(
                    name='bytes_dropped',
                    value=float(bytes_dropped),
                    label='Bytes dropped',
                    levels='levels_bytes_dropped',
                    render_func=render.iobandwidth,
                ),
                VsxMetric(
                    name='bytes_rejected',
                    value=float(bytes_rejected),
                    label='Bytes rejected',
                    levels='levels_bytes_rejected',
                    render_func=render.iobandwidth,
                ),
                VsxMetric(
                    name='packets_accepted',
                    value=float(packets_accepted),
                    label='Packets accepted',
                    levels='levels_packets_accepted',
                    render_func=_render_per_second,
                ),
                VsxMetric(
                    name='packets_dropped',
                    value=float(packets_dropped),
                    label='Packets dropped',
                    levels='levels_packets_dropped',
                    render_func=_render_per_second,
                ),
                VsxMetric(
                    name='packets_rejected',
                    value=float(packets_rejected),
                    label='Packets rejected',
                    levels='levels_packets_rejected',
                    render_func=_render_per_second,
                ),
                VsxMetric(
                    name='packets_processed',
                    value=float(packets_processed),
                    label='Packets processed',
                    levels='levels_packets_processed',
                    render_func=_render_per_second,
                ),
                VsxMetric(
                    name='loggs_send',
                    value=float(loggs_send),
                    label='Logs send',
                    levels='levels_logs_send',
                    render_func=_render_per_second,
                ),
            ],
        )
    return vsx_systems


def discovery_checkpoint_vsx_system(params, section: Dict[str, CheckpointVsx]) -> DiscoveryResult:
    for key in section.keys():
        if section[key].vs_type.lower() in params['vs_type']:
            yield Service(
                item=key,
                parameters={'policyname': section[key].policy_name, 'ha_state': section[key].ha_state}
            )


def check_checkpoint_vsx_system(item, params, section: Dict[str, CheckpointVsx]) -> CheckResult:
    # warn on old params
    if params.get('levels_lower_absolute') or params.get('levels_upper_absolute'):
        yield Result(
            state=State.WARN,
            summary='Reconfigure your "Check Point VSX System" WATO rule(s) please. '
                    'You are using deprecated settings (Maximum/Minimum number of firewall connections)'
        )

    try:
        vsx = section[item]
    except KeyError:
        yield Result(state=State.UNKNOWN, notice='Item not found in SNMP data')
        return

    if not vsx.sic_trust_state.lower() in ['trust established']:
        yield Result(state=State(params['state_sic_not_established']), notice='SIC not established')

    if vsx.vs_type.lower() in ['virtual system', 'vsx gateway']:
        yield Result(state=State.OK, notice=f'System name: {item}')
        try:
            yield Result(state=State.OK, summary=f'Main IP: {ipaddress.ip_address(vsx.main_ip)}')
        except ValueError:
            yield Result(state=State.OK, notice=f'Main IP: {vsx.main_ip}')

        yield Result(state=State.OK, summary=f'VS ID: {vsx.vs_id}', details=f'Virtual system ID: {vsx.vs_id}')
        yield Result(state=State.OK, notice=f'System type: {vsx.vs_type}')

        if not vsx.ha_state.lower() in ['active', 'standby']:
            yield Result(state=State(params['state_ha_not_act_stb']), summary=f'H/A Status: {vsx.ha_state}')
        else:
            yield Result(state=State.OK, summary=f'H/A Status: {vsx.ha_state}')

        if not vsx.policy_type.lower() in ['active']:
            yield Result(state=State(params['state_policy_not_installed']), notice='No policy installed')

        if params['policyname'] != vsx.policy_name:  # policy changed
            yield Result(
                state=State(params['state_policy_changed']),
                notice=f'Policy name changed: expected {params["policyname"]}, found {vsx.policy_name}'
            )

        if params['ha_state'] != vsx.ha_state:  # H/A state changed
            yield Result(
                state=State(params['state_ha_changed']),
                notice=f'State changed: expected/found {params["ha_state"]}/{vsx.ha_state}'
            )

        yield Result(state=State.OK, notice=f'SIC status: {vsx.sic_trust_state}')
        yield Result(state=State.OK, notice=f'Weight: {vsx.vs_weight}')
        yield Result(state=State.OK, notice=f'Policy name: {vsx.policy_name}')
        yield Result(state=State.OK, notice=f'Policy type: {vsx.policy_type}')

        metrics_prefix = 'checkpoint_vsx_'

        # metrics count
        yield from check_levels_predictive(
            value=vsx.connections_active,
            metric_name=f'{metrics_prefix}connections',
            levels=params.get('levels_connections'),
            label='Connections',
            render_func=lambda v: f'{v:.0f}',
            boundaries=(0, None),
        ) if isinstance(params.get('levels_connections'), dict) else check_levels(
            value=vsx.connections_active,
            metric_name=f'{metrics_prefix}connections',
            levels_upper=params.get('levels_connections'),
            # levels_lower=params.get('levels_lower_absolute'),
            label='Connections',
            render_func=lambda v: f'{v:.0f}',
            boundaries=(0, None),
        )

        # metrics rate
        now_time = time.time()
        value_store = get_value_store()

        for metric in vsx.metrics_rate:
            try:
                value = get_rate(
                    value_store,
                    f'{metrics_prefix}{metric.name}',
                    now_time,
                    metric.value,
                    raise_overflow=True,
                )
            except GetRateError:
                continue

            yield from check_levels_predictive(
                value=value,
                metric_name=f'checkpoint_vsx_{metric.name}',
                levels=params.get(metric.levels),
                label=metric.label,
                render_func=metric.render_func,
                boundaries=(0, None),
                # needs patched lib/check_mk/base/api/agent_based/utils.py
                # see PR https://github.com/Checkmk/checkmk/pull/598
                # notice_only=True,
            ) if isinstance(params.get(metric.levels), dict) else check_levels(
                value=value,
                metric_name=f'checkpoint_vsx_{metric.name}',
                levels_upper=params.get(metric.levels),
                # levels_lower=params.get('levels_lower_absolute'),
                label=metric.label,
                render_func=metric.render_func,
                boundaries=(0, None),
                notice_only=True
            )

        yield Metric(value=vsx.connections_peak, name=f'{metrics_prefix}connections_peak')
        if vsx.connections_limit > 0:
            yield Metric(value=vsx.connections_limit, name=f'{metrics_prefix}connections_limit')
    else:
        yield Result(state=State.OK, notice=f'System name: {item}')
        yield Result(state=State.OK, summary=f'Virtual system ID: {vsx.vs_id}')
        yield Result(state=State.OK, summary=f'System Type: {vsx.vs_type}')
        yield Result(state=State.OK, summary=f'SIC status: {vsx.sic_trust_state}')


register.snmp_section(
    name='checkpoint_vsx_system',
    parse_function=parse_checkpoint_vsx_system,
    supersedes=[
        'checkpoint_vsx',
        'checkpoint_vsx_connections',
        'checkpoint_vsx_traffic',
        'checkpoint_vsx_packets',
        'checkpoint_vsx_status',
    ],
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.16',  # CHECKPOINT-MIB::vsx
        oids=[
            '22.1.1.1',   # vsxStatusVSId
            '22.1.1.3',  # vsxStatusVsName
            '22.1.1.4',  # vsxStatusVsType
            '22.1.1.5',  # vsxStatusMainIP
            '22.1.1.6',  # vsxStatusPolicyName
            '22.1.1.7',  # vsxStatusVsPolicyType
            '22.1.1.8',  # vsxStatusSicTrustState
            '22.1.1.9',  # vsxStatusHAState
            '22.1.1.10',  # vsxStatusVSWeight
            '23.1.1.2',  # vsxCountersConnNum
            '23.1.1.3',  # vsxCountersConnPeakNum
            '23.1.1.4',  # vsxCountersConnTableLimit
            '23.1.1.5',  # vsxCountersPackets
            '23.1.1.6',  # vsxCountersDroppedTotal
            '23.1.1.7',  # vsxCountersAcceptedTotal
            '23.1.1.8',  # vsxCountersRejectedTotal
            '23.1.1.9',  # vsxCountersBytesAcceptedTotal
            '23.1.1.10',  # vsxCountersBytesDroppedTotal
            '23.1.1.11',  # vsxCountersBytesRejectedTotal
            '23.1.1.12',  # vsxCountersLoggedTotal
        ]),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_vsx_system',
    service_name='VSX System %s',
    discovery_function=discovery_checkpoint_vsx_system,
    discovery_default_parameters={
        'vs_type': ['virtual system']
    },
    discovery_ruleset_name='discovery_checkpoint_vsx_system',
    check_function=check_checkpoint_vsx_system,
    check_default_parameters={
        'state_sic_not_established': 2,
        'state_ha_not_act_stb': 2,
        'state_policy_not_installed': 2,
        'state_policy_changed': 1,
        'state_ha_changed': 1,

        # 'levels_upper_absolute': (None, None),
        # 'levels_lower_absolute': (None, None),
        # 'policyname': '',
        # 'ha_state': None,
    },
    check_ruleset_name='checkpoint_vsx_system',
)
