#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2018-03-13
#
# Check Point VSX status metrics plugin checkpoint_vsx_system
#

# 2023-06-23: made '/s' unit output non scientific readable numbers
#             set predictive metrics to not auto graph
# 2023-06-26: added definitions for predictive monitoring
#

from cmk.gui.i18n import _

from cmk.gui.plugins.metrics.utils import (
    unit_info,
    metric_info,
    graph_info,
    perfometer_info,
    check_metrics,
)

# make /s unit to use non scientific but readable numbers
# 60.000.000  ->  becomes 60M/s instead of 6e+07/s
from cmk.utils.render import fmt_number_with_precision
from cmk.gui.valuespec import Integer

unit_info["1/s_nonscientific"] = {
    "title": _("per second"),
    "description": _("Frequency (displayed in events/s)"),
    "symbol": _("/s"),
    # "render": lambda v: "{}{}".format(scientific(v, 2), _("/s")),
    "render": lambda v: fmt_number_with_precision(v, drop_zeroes=True, unit="/s"),
    # "js_render": "v => cmk.number_format.scientific(v, 2) + '/s'",
    "js_render": "v => cmk.number_format.fmt_number_with_precision(v, cmk.number_format.SIUnitPrefixes, 2, true)"
                 " + '/s'",
    "stepping": "integer",  # for vertical graph labels
    "valuespec": Integer,
}

metric_info['checkpoint_vsx_connections'] = {
    'title': _('Connections active'),
    'unit': 'count',
    'color': '26/a',
}
metric_info['checkpoint_vsx_connections_peak'] = {
    'title': _('Connections peak'),
    'unit': 'count',
    'color': '21/a',
}
metric_info['checkpoint_vsx_connections_limit'] = {
    'title': _('Connections limit'),
    'unit': 'count',
    'color': '31/a',
}

metric_info['checkpoint_vsx_packets_processed'] = {
    'title': _('Packets processed'),
    'unit': '1/s_nonscientific',
    'color': '13/a',
}
metric_info['checkpoint_vsx_packets_dropped'] = {
    'title': _('Packets dropped'),
    'unit': '1/s_nonscientific',
    'color': '22/a',
}
metric_info['checkpoint_vsx_packets_accepted'] = {
    'title': _('Packets accepted'),
    'unit': '1/s_nonscientific',
    'color': '32/a',
}
metric_info['checkpoint_vsx_packets_rejected'] = {
    'title': _('Packets rejected'),
    'unit': '1/s_nonscientific',
    'color': '42/a',
}

metric_info['checkpoint_vsx_bytes_accepted'] = {
    'title': _('Bytes accepted'),
    'unit': 'bytes/s',
    'color': '16/a',
}
metric_info['checkpoint_vsx_bytes_dropped'] = {
    'title': _('Bytes dropped'),
    'unit': 'bytes/s',
    'color': '23/a',
}
metric_info['checkpoint_vsx_bytes_rejected'] = {
    'title': _('Bytes rejected'),
    'unit': 'bytes/s',
    'color': '33/a',
}

metric_info['checkpoint_vsx_loggs_send'] = {
    'title': _('Loggs send'),
    'unit': '1/s_nonscientific',
    'color': '14/a',
}

metric_info['predict_checkpoint_vsx_connections'] = {
    'title': _('Predicted Connections active'),
    'unit': 'count',
    'color': '26/b',
}

metric_info['predict_checkpoint_vsx_packets_processed'] = {
    'title': _('Predicted Packets processed'),
    'unit': '1/s_nonscientific',
    'color': '13/b',
}
metric_info['predict_checkpoint_vsx_packets_dropped'] = {
    'title': _('Predicted Packets dropped'),
    'unit': '1/s_nonscientific',
    'color': '22/b',
}
metric_info['predict_checkpoint_vsx_packets_accepted'] = {
    'title': _('Predicted Packets accepted'),
    'unit': '1/s_nonscientific',
    'color': '32/b',
}
metric_info['predict_checkpoint_vsx_packets_rejected'] = {
    'title': _('Predicted Packets rejected'),
    'unit': '1/s_nonscientific',
    'color': '42/b',
}

metric_info['predict_checkpoint_vsx_bytes_accepted'] = {
    'title': _('Predicted Bytes accepted'),
    'unit': 'bytes/s',
    'color': '16/b',
}
metric_info['predict_checkpoint_vsx_bytes_dropped'] = {
    'title': _('Predicted Bytes dropped'),
    'unit': 'bytes/s',
    'color': '23/b',
}
metric_info['predict_checkpoint_vsx_bytes_rejected'] = {
    'title': _('Predicted Bytes rejected'),
    'unit': 'bytes/s',
    'color': '33/b',
}

metric_info['predict_checkpoint_vsx_loggs_send'] = {
    'title': _('Predicted Loggs send'),
    'unit': '1/s_nonscientific',
    'color': '14/b',
}


check_metrics['check_mk-checkpoint_vsx_system'] = {
    'checkpoint_vsx_connections_peak': {'auto_graph': False},
    'checkpoint_vsx_connections_limit': {'auto_graph': False},
    # no autograph for predictive monitoring
    'predict_checkpoint_vsx_connections': {'auto_graph': False},
    'predict_checkpoint_vsx_packets_processed': {'auto_graph': False},
    'predict_checkpoint_vsx_packets_dropped': {'auto_graph': False},
    'predict_checkpoint_vsx_packets_accepted': {'auto_graph': False},
    'predict_checkpoint_vsx_packets_rejected': {'auto_graph': False},
    'predict_checkpoint_vsx_bytes_accepted': {'auto_graph': False},
    'predict_checkpoint_vsx_bytes_dropped': {'auto_graph': False},
    'predict_checkpoint_vsx_bytes_rejected': {'auto_graph': False},
    'predict_checkpoint_vsx_loggs_send': {'auto_graph': False},
}

graph_info['checkpoint_vsx_connections'] = {
    'title': _('Check Point VSX: Connections'),
    'metrics': [
        ('checkpoint_vsx_connections', 'area'),
    ],
    'scalars': [
        ('checkpoint_vsx_connections:crit', _('crit')),
        ('checkpoint_vsx_connections:warn', _('warn')),
        ('checkpoint_vsx_connections_limit', _('limit')),
        ('checkpoint_vsx_connections_peak', _('peak')),
    ],
}

graph_info['checkpoint_vsx_packets'] = {
    'title': _('Check Point VSX: Packets'),
    'metrics': [
        ('checkpoint_vsx_packets_rejected', 'stack'),
        ('checkpoint_vsx_packets_dropped', 'stack'),
        ('checkpoint_vsx_packets_accepted', 'stack'),
        ('checkpoint_vsx_packets_processed', 'line'),
    ]
}

graph_info['checkpoint_vsx_bytes'] = {
    'title': _('Check Point VSX: Bytes'),
    'metrics': [
        ('checkpoint_vsx_bytes_rejected', 'stack'),
        ('checkpoint_vsx_bytes_dropped', 'stack'),
        ('checkpoint_vsx_bytes_accepted', 'stack'),
    ]
}

graph_info['checkpoint_vsx_loggs_send'] = {
    'title': _('Check Point VSX: Logs'),
    'metrics': [
        ('checkpoint_vsx_loggs_send', 'area'),
    ]
}

perfometer_info.append(('stacked', [
    {
        'type': 'logarithmic',
        'metric': 'checkpoint_vsx_connections',
        'half_value': 1000000.0,
        'exponent': 2,
    },
    {
        'type': 'logarithmic',
        'metric': 'checkpoint_vsx_bytes_accepted',
        'half_value': 10000000.0,
        'exponent': 2,
    },
]))
