#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-09-07
#

# 2023-06-17: changed upper/lower levels for active connections to levels (predictive)
#             added wato levels option for bytes/packets perf counters
# 2023-06-18: moved wato file to check_parameters sub directory

from cmk.gui.i18n import _
from cmk.gui.valuespec import (
    Dictionary,
    TextAscii,
    MonitoringState,
    ListChoice,
    TextUnicode,
)

from cmk.gui.plugins.wato.utils import (
    CheckParameterRulespecWithItem,
    rulespec_registry,
    RulespecGroupCheckParametersNetworking,
    RulespecGroupCheckParametersDiscovery,
    HostRulespec,
    Levels,
)


def _parameter_valuespec_checkpoint_vsx_system():
    return Dictionary(
        elements=[
            ('state_ha_not_act_stb',
             MonitoringState(
                 title=_('State if H/A state is not active/standby'),
                 help=_('Monitoring state if H/A state is not active or standby. Defaulr is "CRIT"'),
                 default_value=2,
             )),
            ('state_ha_changed',
             MonitoringState(
                 title=_('State if H/A state has changed'),
                 help=_('Monitoring state if H/A state has changed. Default is "WARN".'),
                 default_value=1,
             )),
            ('state_policy_not_installed',
             MonitoringState(
                 title=_('State if no policy is installed'),
                 help=_('Monitoring state if no policy is installed. Default is "CRIT"'),
                 default_value=2,
             )),
            ('state_policy_changed',
             MonitoringState(
                 title=_('State if policy name has changed'),
                 help=_('Monitoring status on policy name change. Default is "WARN".'),
                 default_value=1,
             )),
            ('state_sic_not_established',
             MonitoringState(
                 title=_('State if SIC is not established'),
                 help=_(
                     'Monitoring state if SIC (Secure Internal Communication) is not established. Default is "CRIT"'
                 ),
                 default_value=2,
             )),
            ('levels_bytes_accepted', Levels(title=_('Levels for Bytes Accepted'), unit=_('Bytes/s'), )),
            ('levels_bytes_dropped', Levels(title=_('Levels for Bytes Dropped'), unit=_('Bytes/s'), )),
            ('levels_bytes_rejected', Levels(title=_('Levels for Bytes Rejected'), unit=_('Bytes/s'), )),
            ('levels_connections', Levels(title=_('Levels for Connections'), unit=_('Connections'), )),
            ('levels_logs_send', Levels(title=_('Levels for Logs Send'), unit=_('Logs/s'), )),
            ('levels_packets_accepted', Levels(title=_('Levels for Packets Accepted'), unit=_('Packets/s'), )),
            ('levels_packets_dropped', Levels(title=_('Levels for Packets Dropped'), unit=_('Packets/s'), )),
            ('levels_packets_rejected', Levels(title=_('Levels for Packets Rejected'), unit=_('Packets/s'), )),
            ('levels_packets_processed', Levels(title=_('Levels for Packets Processed'), unit=_('Packets/s'), )),
            # added by plugin discovery function -> hidden key
            ('policyname', TextUnicode()),
            ('ha_state', TextUnicode()),
        ],
        hidden_keys=['policyname', 'ha_state'],
        ignored_keys=['levels_upper_absolute', 'levels_lower_absolute'],
        # shows help before an item is activated, nur bei render "normal"
        # columns=2,
        # space saver -> Title to the Right -> item starts on same line. Adds collapsed properties
        # form_part like form, only more space between label and checkbox
        # render='form',
        # item on same hight as label, is multiline item label moves to the middle of item -> saves space
        # render form looks better
        # form_narrow=True,
        # same as form_narrow=True (?)
        # form_isopen=False,
    )


rulespec_registry.register(
    CheckParameterRulespecWithItem(
        check_group_name='checkpoint_vsx_system',
        group=RulespecGroupCheckParametersNetworking,
        match_type='dict',
        parameter_valuespec=_parameter_valuespec_checkpoint_vsx_system,
        title=lambda: _('Check Point VSX system'),
        item_spec=lambda: TextAscii(title=_('VSX System name'), ),
    ))


def _valuespec_discovery_checkpoint_vsx_system():
    _vs_types = [
        ('virtual system', 'Virtual System'),
        ('vsx gateway', 'VSX Gateway'),
        ('virtual switch', 'Virtual Switch'),
        ('virtual router', 'Virtual Router'),
    ]
    return Dictionary(
            title=_('Check Point VSX system'),
            elements=[
                ('vs_type',
                 ListChoice(
                     title=_('VS types to discover'),
                     help=_('Virtual system types to discover. Note: if you select "VSX Gateway", '
                            'this will also discover ClusterXL systems.'),
                     choices=_vs_types,
                     default_value=[
                         'virtual system',
                     ],
                 )),
            ],
        )


rulespec_registry.register(
    HostRulespec(
        group=RulespecGroupCheckParametersDiscovery,
        match_type='dict',
        name='discovery_checkpoint_vsx_system',
        valuespec=_valuespec_discovery_checkpoint_vsx_system,
    ))
